/*
 * conflictcheckonly.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */
#include <string>
#include <list>
#include <vdr/plugin.h>
#include <vdr/i18n.h>
#include "services.h"

static const char VERSION[]        = "0.0.1";
static const char DESCRIPTION[]    = "Direct access to epgsearch's conflict check menu";
static const char MAINMENUENTRY[]  = "Timer conflicts";

int showInfoInMain=1;
char mainmenuentry[MaxFileName] = "";

const tI18nPhrase Phrases[] = {
  { "Direct access to epgsearch's conflict check menu",
    "Direkter Zugriff auf epgsearch's Konflikt-Prfungs-Men",// Deutsch
    "",// TODO 
    "",// TODO
    "",// TODO
    "",// TODO 
    "",// TODO
    "",// TODO 
    "Suoratoiminto EPGSearch-laajennoksen ajastimien tarkistukselle",
    "",// TODO Polski
    "",// TODO Espaol
    "",// TODO  (Greek)
    "",// TODO Svenska
    "",// TODO Romaneste
    "",// TODO Magyar
    "",// TODO Catal
    "",// TODO  (Russian)
    "",// TODO Hrvatski (Croatian)
    "", // Eesti
    "", // Dansk
    "", // Czech
  },
  { "Timer conflicts",
    "Timer-Konflikte",
    "",// TODO
    "",// TODO
    "",
    "",// TODO
    "",
    "",// TODO
    "Ajastimien pllekkisyydet",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "", // Eesti
    "", // Dansk
    "", // Czech
  },
  { "This version of EPGSearch does not support this service!",
    "Diese Version von EPGSearch untersttzt diesen Dienst nicht!",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "EPGSearch-laajennos ei tarjoa vaadittavaa palvelua!",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "", // Eesti
    "", // Dansk
    "", // Czech
  },
  { "EPGSearch does not exist!",
    "EPGSearch nicht vorhanden!",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "EPGSearch-laajennosta ei lydy!",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "", // Eesti
    "", // Dansk
    "", // Czech
  },
  { "Plugin needs vdr-1.3.36 or above!",
    "Plugin bentigt vdr-1.3.36 oder hher!",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "Laajennos tarvitsee toimiakseen vhintn vdr-1.3.36!",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "", // Eesti
    "", // Dansk
    "", // Czech
  },
  { "next",
    "nchster",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "seuraava",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "", // Eesti
    "", // Dansk
    "", // Czech
  },
  { "Show conflict info in main menu entry",
    "Zeige Konflikt-Info in Hauptmeneintrag",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "Nyt pllekkisyydet pvalikossa",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "", // Eesti
    "", // Dansk
    "", // Czech
  },
  { NULL }
  };
  
cString DateTime(time_t t)
{
  char buffer[32];
  if (t == 0)
     time(&t);
  struct tm tm_r;
  tm *tm = localtime_r(&t, &tm_r);
  snprintf(buffer, sizeof(buffer), "%02d.%02d %02d:%02d", tm->tm_mday, tm->tm_mon + 1, tm->tm_hour, tm->tm_min);
  return buffer;
}

class cMenuConflictcheckonlySetup : public cMenuSetupPage {
  private:
    virtual void Setup(void)
	{
	    Add(new cMenuEditBoolItem(tr("Show conflict info in main menu entry"), &showInfoInMain, tr("no"), tr("yes")));
	}
  protected:
    virtual void Store(void)
	{
	    SetupStore("ConflictInfoInMain", showInfoInMain);
	}
  public:
    cMenuConflictcheckonlySetup(void)
	{
	    Setup();
	};
  };


class cPluginConflictcheckonly : public cPlugin {
private:
  // Add any member variables or functions you may need here.
public:
  cPluginConflictcheckonly(void);
  virtual ~cPluginConflictcheckonly();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return tr(DESCRIPTION); }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Stop(void);
  virtual void Housekeeping(void);
  virtual const char *MainMenuEntry(void);
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  };

cPluginConflictcheckonly::cPluginConflictcheckonly(void)
{
}

cPluginConflictcheckonly::~cPluginConflictcheckonly()
{
}

const char* cPluginConflictcheckonly::MainMenuEntry(void)
{
    if (showInfoInMain)
    {
	Epgsearch_lastconflictinfo_v1_0* serviceData = new Epgsearch_lastconflictinfo_v1_0;
	serviceData->nextConflict = 0;
	serviceData->relevantConflicts = 0;
	serviceData->totalConflicts = 0;
	
	cPlugin *p = cPluginManager::GetPlugin("epgsearch");
	if (p)
	    p->Service("Epgsearch-lastconflictinfo-v1.0", serviceData);
	
	if (serviceData->relevantConflicts > 0)
	{
	    sprintf(mainmenuentry, "%s (%d, %s: %s)", tr(MAINMENUENTRY), serviceData->relevantConflicts, 
		    tr("next"), *DateTime(serviceData->nextConflict));
	    delete serviceData;
	    return mainmenuentry;
	}
	
	delete serviceData;
    }
    return tr(MAINMENUENTRY);
}

const char *cPluginConflictcheckonly::CommandLineHelp(void)
{
  return NULL;
}

bool cPluginConflictcheckonly::ProcessArgs(int argc, char *argv[])
{
  return true;
}

bool cPluginConflictcheckonly::Initialize(void)
{
  RegisterI18n(Phrases);
  return true;
}

bool cPluginConflictcheckonly::Start(void)
{
  return true;
}

void cPluginConflictcheckonly::Stop(void)
{
}

void cPluginConflictcheckonly::Housekeeping(void)
{
}

cOsdObject *cPluginConflictcheckonly::MainMenuAction(void)
{
    cPlugin *p = cPluginManager::GetPlugin("epgsearch");
    if (p)
    {
 	Epgsearch_conflictmenu_v1_0* serviceData = new Epgsearch_conflictmenu_v1_0;
	serviceData->pConflictMenu = NULL;
	
	cOsdMenu* pMenu = NULL;
	if (p->Service("Epgsearch-conflictmenu-v1.0", serviceData))
	    pMenu = serviceData->pConflictMenu;
	else
	    Skins.Message(mtError, tr("This version of EPGSearch does not support this service!"));
	
	delete serviceData;
	
	if (pMenu)
	    return pMenu;	
	else
	    return NULL;
    }
    else
    {
	Skins.Message(mtError, tr("EPGSearch does not exist!"));
	return NULL;
    }

  return NULL;
}

cMenuSetupPage *cPluginConflictcheckonly::SetupMenu(void)
{
  return new cMenuConflictcheckonlySetup;
}

bool cPluginConflictcheckonly::SetupParse(const char *Name, const char *Value)
{
  if (!strcasecmp(Name, "ConflictInfoInMain")) showInfoInMain = atoi(Value);
  return true;
}

VDRPLUGINCREATOR(cPluginConflictcheckonly); // Don't touch this!
