#include <set>
#include "menu_favorites.h"
#include "menu_whatson.h"
#include "menu_commands.h"

using std::set;

extern const char* ButtonBlue[3];
extern int exitToMainMenu;

cMenuFavorites::cMenuFavorites()
    : cMenuSearchResults(cTemplFile::GetTemplateByName("MenuFavorites"))
{
    BuildList();
}

bool cMenuFavorites::BuildList(bool)
{
    Clear();

    cSearchResults* pCompleteSearchResults = NULL;
    cMutexLock SearchExtsLock(&SearchExts);
    cSearchExt *SearchExt = SearchExts.First();
    int timespan = EPGSearchConfig.FavoritesMenuTimespan*60;

    while (SearchExt) 
    {
	if (SearchExt->useInFavorites)
	    pCompleteSearchResults = SearchExt->Run(modeBlue == showNoPayTV?1:0, false, timespan, pCompleteSearchResults, true);
	SearchExt = SearchExts.Next(SearchExt);
    }

    if (pCompleteSearchResults)
    {
	set<const cEvent*> foundEvents;
	pCompleteSearchResults->SortBy(CompareEventTime);
	
	for (cSearchResult* pResultObj = pCompleteSearchResults->First(); 
	     pResultObj; 
	     pResultObj = pCompleteSearchResults->Next(pResultObj))
	{
	    if (foundEvents.find(pResultObj->event) == foundEvents.end())
	    {
		foundEvents.insert(pResultObj->event);
		Add(new cMenuSearchResultsItem(pResultObj->event, modeYellow == showEpisode, false, menuTemplate));
	    }
	}	
	delete pCompleteSearchResults;
    }
    SetHelpKeys();  
    char* szTitle = NULL;
    asprintf(&szTitle, "%s: %d %s", tr("Favorites"), Count(), tr("Search results"));
    SetTitle(szTitle);
    free(szTitle);
    Display();
    
    return true;
}

eOSState cMenuFavorites::ProcessKey(eKeys Key)
{
    exitToMainMenu = 0;
    if (!HasSubMenu() && Key == kBack)
    {
	exitToMainMenu = 1;
	return osBack;
    }

    eOSState state = cOsdMenu::ProcessKey(Key);
    if (state == osUnknown) 
    {
	switch (Key) {
	    case kFastRew:
		if(HasSubMenu()) 	     
		{
		    CursorUp();
		    return ShowSummary();
		}
		break;
	    case kFastFwd:
		if(HasSubMenu()) 	     
		{
		    CursorDown();
		    return ShowSummary();
		}
		break;
	    case kRecord:
	    case kRed:
		if(HasSubMenu()) {
		    state = Record();
		    break;
		} 
		if (Count())
		{
		    if (EPGSearchConfig.redkeymode==toggleKeys)
			state = Record();
		    else
		    {
			cMenuSearchResultsItem* mi = (cMenuSearchResultsItem*)Get(Current());
			if (mi) {
			    if (mi->event) {
				return AddSubMenu(new cMenuSearchCommands(tr("EPG Commands"),mi->event, false, searchExt));
			    }
			}
		    }
		}
		break;
	    case k0:
		if(!HasSubMenu()) 
		{
		    toggleKeys = 1 - toggleKeys;
		    SetHelpKeys();
		}
		state = osContinue;
		break;
	    case k1...k9: 
		if (!HasSubMenu())
		    return Commands(Key);
		else
		    state = osContinue;
		break;
	    case kGreen:  
		if(HasSubMenu()) 	     
		{
		    CursorUp();
		    return ShowSummary();
		}
		else
		{
		    toggleKeys = 0;
		    cMenuWhatsOnSearch::currentShowMode = cMenuWhatsOnSearch::GetNextMode();
		    return osUnknown;
		}
		break;
	    case kYellow:  
		if(HasSubMenu()) 	     
		{
		    CursorDown();
		    return ShowSummary();
		}
		else
		{
		    cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
		    if (item && item->event)
		    {
			cChannel *channel = Channels.GetByChannelID(item->event->ChannelID(), true);
			cMenuWhatsOnSearch::scheduleChannel = channel;
			cMenuWhatsOnSearch::currentShowMode = showNow;
		    }
		    toggleKeys = 0;
		    return osBack;
		}
		break;
	    case kBlue:
		return EPGSearchConfig.useOkForSwitch?ShowSummary():Switch();
		break;
	    case kOk:
		if(HasSubMenu()) 
		{
		    state = cOsdMenu::ProcessKey(Key);
		    break;
		}
		if (Count())
		    return EPGSearchConfig.useOkForSwitch?Switch():ShowSummary();
		else
		    state = osBack;
		break;
	    default:      
		break;
	}
    }
    return state;
}

void cMenuFavorites::SetHelpKeys()
{
    cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
    bool hasTimer = item && item->Selectable() && item->timerMatch == tmFull;

    showMode nextShowMode = cMenuWhatsOnSearch::GetNextMode();    
    cShowMode* mode = cMenuWhatsOnSearch::GetShowMode(nextShowMode);
    char* szButtonGreen = NULL;
    if (mode) 
	szButtonGreen = mode->GetDescription();

    if (toggleKeys==0)
	SetHelp((EPGSearchConfig.redkeymode==0?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), szButtonGreen,tr("Button$Schedule"), EPGSearchConfig.useOkForSwitch?tr("Button$Info"):tr("Button$Switch"));
    else
	SetHelp((EPGSearchConfig.redkeymode==1?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), szButtonGreen,tr("Button$Schedule"), EPGSearchConfig.useOkForSwitch?tr("Button$Info"):tr("Button$Switch"));
}

