/*
 * menu_searchresults.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <vector>
#include "menu_searchresults.h"
#include "menu_myedittimer.h"
#include "menu_event.h"
#include "menu_commands.h"
#include "epgsearchcfg.h"
#include "epgsearchtools.h"
#include "changrp.h"
#include "recdone.h"
#include "epgsearchcats.h"
#include <vdr/menu.h>
#include "menu_conflictcheck.h"
#include "uservars.h"
#include "menu_deftimercheckmethod.h"

const char* ButtonBlue[3];
extern int gl_InfoConflict;

static int CompareRecording(const void *p1, const void *p2)
{
  return (int)((*(cRecording **)p1)->start - (*(cRecording **)p2)->start);
}

// --- cMenuSearchResultsItem -------------------------------------------------------
cMenuSearchResultsItem::cMenuSearchResultsItem(const cEvent *EventInfo, bool EpisodeOnly, bool PreviewTimer, cMenuTemplate* MenuTemplate)
{
    fileName = NULL;
    event = EventInfo;
    timerMatch = tmNone;
    episodeOnly = EpisodeOnly;
    previewTimer = PreviewTimer;
    menuTemplate = MenuTemplate?MenuTemplate:cTemplFile::GetTemplateByName("MenuSearchResults");
	
    Update(true);
}

bool cMenuSearchResultsItem::Update(bool Force)
{
    if (!menuTemplate)
	return false;
    
    bool result = false;

    int OldTimerMatch = timerMatch;
    bool hasMatch = false; 
    cTimer* timer = NULL;
    if (event) timer = Timers.GetMatch(event, &timerMatch);
    if (timer) hasMatch = true;
    
    if (Force || timerMatch != OldTimerMatch) 
    {
	char t[2]="",v[2]="",r[2]="";
	char szStatus[4] = "";
	if (EPGSearchConfig.WarEagle)
	{
	    sprintf(t, "%c", event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?249:253) : 't' : ' ');
	    sprintf(v, "%c", event && event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ');
	    sprintf(r, "%c", event && event->IsRunning() ? 251 : ' ');
	}
	else
	{
	    sprintf(t, "%c", event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?'R':'T') : 't' : ' ');
	    sprintf(v, "%c", event && event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ');
	    sprintf(r, "%c", event && event->IsRunning() ? '*' : ' ');
	}
	if (t[0] != 'T' && previewTimer)
	    t[0] = 'P';
	
	sprintf(szStatus, "%s%s%s", t,v,r);
	
	char* buffer = strdup(menuTemplate->MenuTemplate());
	char* tmp = strreplaceall(buffer, '|', "\t");
	free(buffer);
	buffer = tmp;

	if (!strcasestr(buffer, "%subtitle%")) // make sure, there is a subtitle
	    buffer = strreplacei(buffer, "%title%", "%title% ~ %subtitle%");
	if (episodeOnly)
	    buffer = strreplacei(buffer, "%title%", "");
	    
	// parse the epxression and evaluate it
	cVarExpr varExpr(buffer);
	tmp = strdup(varExpr.Evaluate(event).c_str());
	free(buffer);
	buffer = tmp;

	buffer = strreplacei(buffer, "%status%", szStatus);
	buffer = strreplacei(buffer, "%t_status%", t);
	buffer = strreplacei(buffer, "%v_status%", v);
	buffer = strreplacei(buffer, "%r_status%", r);

	buffer = FixSeparators(buffer, '~');
	buffer = FixSeparators(buffer, ':');
	buffer = FixSeparators(buffer, '-');
	
	SetText(buffer, false);

	if (EPGSearchConfig.checkTimerConflAfterTimerProg && !Force && timer && timerMatch && timerMatch != OldTimerMatch)
	{
	    cConflictCheck C;
	    C.Check();
	    if (C.TimerInConflict(timer))
		gl_InfoConflict = 1;	  
	}

	return true;
    }
    return result;
}

cMenuSearchResultsItem::cMenuSearchResultsItem(cRecording *Recording)
{
  fileName = strdup(Recording->FileName());
  SetText(Recording->Title('\t'));
}

// --- cMenuSearchResults -------------------------------------------------------

const cEvent *cMenuSearchResults::scheduleEventInfo = NULL;

cMenuSearchResults::cMenuSearchResults(const char *p2, cMenuTemplate* MenuTemplate, bool bSearchInRecordings)
:cOsdMenu(bSearchInRecordings ? tr("found recordings") : tr("additional starting times"), MenuTemplate->Tab(0), MenuTemplate->Tab(1), MenuTemplate->Tab(2), MenuTemplate->Tab(3), MenuTemplate->Tab(4))
{
    ButtonBlue[0] = strdup(tr("Button$all channels"));
    ButtonBlue[1] = strdup(tr("Button$only FTA"));
    ButtonBlue[2] = strdup(tr("Button$Timer preview"));

    // create a dummy search
    asprintf(&m_pSearchString, "%s", p2);
    searchExt = new cSearchExt;
    strcpy(searchExt->search, m_pSearchString);
    searchExt->mode = 0; // substring
    searchExt->useTitle = 1;
    searchExt->useSubtitle = 0;
    searchExt->useDescription = 0;

    blacklist = NULL;
    m_bSort = true;
    m_bSearchInRecordings = bSearchInRecordings;
    modeYellow = showTitleEpisode;
    modeBlue = EPGSearchConfig.ignorePayTV?showNoPayTV:showAll;
    menuTemplate = MenuTemplate;
    
    if (bSearchInRecordings)
	ScanRecordings();
    else
	BuildList(true);
}

cMenuSearchResults::~cMenuSearchResults()
{
    if (m_pSearchString)
    {
	delete searchExt;
	free(m_pSearchString);
    }
}

cMenuSearchResults::cMenuSearchResults(cSearchExt* SearchExt, cMenuTemplate* MenuTemplate)
    :cOsdMenu(tr("Search results"), MenuTemplate->Tab(0), MenuTemplate->Tab(1), MenuTemplate->Tab(2), MenuTemplate->Tab(3), MenuTemplate->Tab(4))
{
    ButtonBlue[0] = strdup(tr("Button$all channels"));
    ButtonBlue[1] = strdup(tr("Button$only FTA"));
    ButtonBlue[2] = strdup(tr("Button$Timer preview"));

    searchExt = SearchExt;
    blacklist = NULL;
    m_pSearchString = NULL;
    m_bSort = true;
    m_bSearchInRecordings = false;
    modeYellow = showTitleEpisode;
    modeBlue = searchExt->useChannel==3?showNoPayTV:(EPGSearchConfig.ignorePayTV?showNoPayTV:showAll);
    menuTemplate = MenuTemplate;
   
    BuildList();
}

cMenuSearchResults::cMenuSearchResults(cBlacklist* Blacklist, cMenuTemplate* MenuTemplate)
    :cOsdMenu(tr("Blacklist results"), MenuTemplate->Tab(0), MenuTemplate->Tab(1), MenuTemplate->Tab(2), MenuTemplate->Tab(3), MenuTemplate->Tab(4))
{
    ButtonBlue[0] = strdup(tr("Button$all channels"));
    ButtonBlue[1] = strdup(tr("Button$only FTA"));
    ButtonBlue[2] = strdup(tr("Button$Timer preview"));

    blacklist = Blacklist;
    searchExt = NULL;
    m_pSearchString = NULL;
    m_bSort = true;
    m_bSearchInRecordings = false;
    modeYellow = showTitleEpisode;
    modeBlue = blacklist->useChannel==3?showNoPayTV:(EPGSearchConfig.ignorePayTV?showNoPayTV:showAll);
    menuTemplate = MenuTemplate;
   
    BuildBlacklist();
}

cMenuSearchResults::cMenuSearchResults(cMenuTemplate* MenuTemplate)
    :cOsdMenu("", MenuTemplate->Tab(0), MenuTemplate->Tab(1), MenuTemplate->Tab(2), MenuTemplate->Tab(3), MenuTemplate->Tab(4))
{
    menuTemplate = MenuTemplate;
    searchExt = NULL;
    blacklist = NULL;
    m_pSearchString = NULL;
    m_bSearchInRecordings = false;
    modeYellow = showTitleEpisode;
}

int cMenuSearchResults::GetTab(int Tab)
{
    if (!menuTemplate)
	menuTemplate = cTemplFile::GetTemplateByName("MenuSearchResults");

    return menuTemplate->Tab(Tab-1);
}

bool cMenuSearchResults::BuildList(bool IgnoreRunning)
{
    if (searchExt) return BuildSearchlist(IgnoreRunning);
    if (blacklist) return BuildBlacklist(IgnoreRunning);
    return false;
}

bool cMenuSearchResults::BuildSearchlist(bool IgnoreRunning)
{
    time_t now = time(NULL);
    cSearchResults* pSearchResults = searchExt->Run(modeBlue == showNoPayTV?1:0, false, 0, NULL, modeBlue != showTimerPreview);
    Clear();
    if (pSearchResults)
    {
	pSearchResults->SortBy(m_bSort? CompareEventTime: CompareEventChannel);
	
	for (cSearchResult* pResultObj = pSearchResults->First(); 
	     pResultObj; 
	     pResultObj = pSearchResults->Next(pResultObj))
	{
	    if (IgnoreRunning && now > pResultObj->event->StartTime())
		continue;
	    if (!(searchExt->useAsSearchTimer && searchExt->avoidRepeats && modeBlue == showTimerPreview))
		pResultObj->needsTimer = false;
	    
	    Add(new cMenuSearchResultsItem(pResultObj->event, modeYellow == showEpisode, pResultObj->needsTimer, menuTemplate));
	}
	
	delete pSearchResults;
    }
    if (Count())
	SetCurrent(Get(0));
    SetHelpKeys();  
    char* szTitle = NULL;
    asprintf(&szTitle, "%d %s - %s", Count(), tr("Search results"), searchExt->search);
    SetTitle(szTitle);
    free(szTitle);
    Display();
    
    return true;
}

bool cMenuSearchResults::BuildBlacklist(bool IgnoreRunning)
{
    time_t now = time(NULL);
    cSearchResults* pSearchResults = blacklist->Run();
    Clear();
    if (pSearchResults)
    {
	pSearchResults->SortBy(m_bSort? CompareEventTime: CompareEventChannel);
	
	for (cSearchResult* pResultObj = pSearchResults->First(); 
	     pResultObj; 
	     pResultObj = pSearchResults->Next(pResultObj))
	{
	    if (IgnoreRunning && now > pResultObj->event->StartTime())
		continue;
	    Add(new cMenuSearchResultsItem(pResultObj->event, modeYellow == showEpisode, false));
	}
	
	delete pSearchResults;
    }
    if (Count())
	SetCurrent(Get(0));
    SetHelpKeys();  
    char* szTitle = NULL;
    asprintf(&szTitle, "%d %s - %s", Count(), tr("Blacklist results"), blacklist->search);
    SetTitle(szTitle);
    free(szTitle);
    Display();
    
    return true;
}

bool cMenuSearchResults::Update(void)
{
    if (m_bSearchInRecordings) return true;
    bool result = false;
    for (cOsdItem *item = First(); item; item = Next(item)) {
	if (((cMenuSearchResultsItem *)item)->Update())
	    result = true;
    }
    return result;
}

bool cMenuSearchResults::ScanRecordings(void)
{
    cRecording **pArray = NULL;
    int num = 0;
    
    Clear();
    for (cRecording *recording = Recordings.First(); recording; recording = Recordings.Next(recording)) {
	if (strstr(recording->Title(), m_pSearchString) ||
	    strstr(m_pSearchString, recording->Title())) {
	    pArray = (cRecording **)realloc(pArray, (num + 1) * sizeof(cRecording *));
	    pArray[num++] = recording;
	}
    }

    qsort(pArray, num, sizeof(cRecording *), CompareRecording);

    for (int a = 0; a < num; a++)
	Add(new cMenuSearchResultsItem(pArray[a]));

    delete pArray;

    SetHelp(NULL);
    Display();
    return true;
}

void cMenuSearchResults::SetHelpKeys()
{
    cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
    bool hasTimer = item && item->Selectable() && item->timerMatch == tmFull;

    ModeBlueSR nextModeBlue = (ModeBlueSR)(((int)modeBlue+1)%3);
    if (nextModeBlue == showTimerPreview && 
	(!searchExt || (searchExt && (searchExt->useAsSearchTimer == 0 || searchExt->avoidRepeats == 0))))
	nextModeBlue = (ModeBlueSR)(((int)nextModeBlue+1)%3);

    if (toggleKeys==0)
	SetHelp((EPGSearchConfig.redkeymode==0?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), blacklist?NULL:ButtonBlue[(int)nextModeBlue]);
    else
	SetHelp((EPGSearchConfig.redkeymode==1?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), m_bSort? tr("Button$by channel"):tr("Button$by time"), modeYellow==showTitleEpisode?tr("Button$Episode"):tr("Button$Title"), blacklist?NULL:ButtonBlue[(int)nextModeBlue]);
}

cRecording *cMenuSearchResults::GetRecording(cMenuSearchResultsItem *Item)
{
    cRecording *recording = Recordings.GetByName(Item->FileName());
    if (!recording)
	Skins.Message(mtError, tr("Error while accessing recording!"));
    return recording;
}

eOSState cMenuSearchResults::Play(void)
{
  cMenuSearchResultsItem *ri = (cMenuSearchResultsItem*)Get(Current());
  if (ri) 
  {
      cRecording *recording = GetRecording(ri);
      if (recording) {
	  cReplayControl::SetRecording(recording->FileName(), recording->Title());
	  return osReplay;
      }
  }

  return osContinue;
}

eOSState cMenuSearchResults::Record(void)
{
  cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
  if (item) {
      if (item->timerMatch == tmFull) 
      {
	  int tm = tmNone;
	  cTimer *timer = Timers.GetMatch(item->event, &tm);
	  if (timer)
	      if (EPGSearchConfig.useVDRTimerEditMenu)
		  return AddSubMenu(new cMenuEditTimer(timer));
	      else
		  return AddSubMenu(new cMenuMyEditTimer(timer, false, item->event));
      }

     cTimer *timer = new cTimer(item->event);
     PrepareTimerFile(item->event, timer);
     cTimer *t = Timers.GetTimer(timer);
     if (EPGSearchConfig.onePressTimerCreation == 0 || t || !item->event || (!t && item->event && item->event->StartTime() - (Setup.MarginStart+2) * 60 < time(NULL))) 
     {
	 if (t)
	 {
	     delete timer;
	     timer = t;      
	 }
	 if (EPGSearchConfig.useVDRTimerEditMenu)
	     return AddSubMenu(new cMenuEditTimer(timer, !t));
	 else
	     return AddSubMenu(new cMenuMyEditTimer(timer, !t, item->event));     
     }
     else
     {
	 string fullaux = "";
	 if (item->event)
	 {
	     const cEvent* event = item->event;
	     int bstart = event->StartTime() - timer->StartTime();
	     int bstop = timer->StopTime() - event->EndTime();
	     string aux = "";
	     int checkmode = DefTimerCheckModes.GetMode(timer->Channel());
	     aux = UpdateAuxValue(aux, "update", checkmode);
	     aux = UpdateAuxValue(aux, "eventid", event->EventID());
	     aux = UpdateAuxValue(aux, "bstart", bstart);
	     aux = UpdateAuxValue(aux, "bstop", bstop);
	     fullaux = UpdateAuxValue(fullaux, "epgsearch", aux);
	 }
	 SetAux(timer, fullaux);
	 Timers.Add(timer);
	 timer->Matches();
	 Timers.SetModified();
	 LogFile.iSysLog("timer %s added (active)", *timer->ToDescr());

	 if (HasSubMenu())
	     CloseSubMenu();
	 if (Update())
	     Display();
	 SetHelpKeys();
     }
  }
  return osContinue;
}


eOSState cMenuSearchResults::Switch(void)
{
  cMenuSearchResultsItem *item = (cMenuSearchResultsItem *)Get(Current());
  if (item) {
     cChannel *channel = Channels.GetByChannelID(item->event->ChannelID(), true);
     if (channel && cDevice::PrimaryDevice()->SwitchChannel(channel, true))
        return osEnd;
     }
  Skins.Message(mtInfo, tr("Can't switch channel!")); 
  return osContinue;
}

eOSState cMenuSearchResults::Commands(eKeys Key)
{
  if (HasSubMenu() || Count() == 0)
     return osContinue;
  cMenuSearchResultsItem *mi = (cMenuSearchResultsItem *)Get(Current());
  if (mi && mi->event) {
        cMenuSearchCommands *menu;
        eOSState state = AddSubMenu(menu = new cMenuSearchCommands(tr("EPG Commands"), mi->event, true, searchExt));
        if (Key != kNone)
           state = menu->ProcessKey(Key);
        return state;
     }
  return osContinue;
}

eOSState cMenuSearchResults::ShowSummary()
{
    if (Count())
    {
	const cEvent *ei = ((cMenuSearchResultsItem *)Get(Current()))->event;
	if (ei) 
	{
	    cChannel *channel = Channels.GetByChannelID(ei->ChannelID(), true);
	    if (channel)
		return AddSubMenu(new cMenuEventSearch(ei, channel->Number() != cDevice::CurrentChannel()));
	}
    }
    return osContinue;
}

eOSState cMenuSearchResults::ProcessKey(eKeys Key)
{
    bool HadSubMenu = HasSubMenu();
    eOSState state = cOsdMenu::ProcessKey(Key);

    if (state == osUnknown) {
	if (!m_bSearchInRecordings)
	{
	    switch (Key) {
		case kFastRew:
		    if(HasSubMenu()) 	     
		    {
			CursorUp();
			return ShowSummary();
		    }
		    break;
		case kFastFwd:
		    if(HasSubMenu()) 	     
		    {
			CursorDown();
			return ShowSummary();
		    }
		    break;
		case kRecord:
		case kRed:
		    if(HasSubMenu()) {
			state = Record();
			break;
		    } 
		    if (Count())
		    {
			if (EPGSearchConfig.redkeymode==toggleKeys)
			    state = Record();
			else
			{
			    cMenuSearchResultsItem* mi = (cMenuSearchResultsItem*)Get(Current());
			    if (mi) {
				if (mi->event) {
				    return AddSubMenu(new cMenuSearchCommands(tr("EPG Commands"),mi->event, false, searchExt));
				}
			    }
			}
		    }
		    break;
		case k0:
		    if(!HasSubMenu()) 
		    {
			toggleKeys = 1 - toggleKeys;
			SetHelpKeys();
		    }
		    state = osContinue;
		    break;
		case k1...k9: 
		    if (!HasSubMenu())
			return Commands(Key);
		    else
			state = osContinue;
		    break;
		case kGreen:  {
		    if(HasSubMenu()) 	     
		    {
			CursorUp();
			return ShowSummary();
		    }
		    else
		    {
			m_bSort=!m_bSort;
			BuildList();
			state = osContinue;
		    }
		}
		    break;
		case kBlue:
		    if (HasSubMenu())
			return Switch();
		    else
		    {
			if (!blacklist)
			{
			    modeBlue = (ModeBlueSR)(((int)modeBlue+1)%3);
			    if (modeBlue == showTimerPreview && 
				(!searchExt || (searchExt && (searchExt->useAsSearchTimer == 0 || searchExt->avoidRepeats == 0))))
				modeBlue = (ModeBlueSR)(((int)modeBlue+1)%3);
			    
			    if (modeBlue == showTimerPreview)
				m_bSort = true; // show always sorted by channel
			    BuildList();
			}
			state = osContinue;			
		    }
		    break;
		case kYellow: 
		    if(HasSubMenu()) 	     
		    {
			CursorDown();
			return ShowSummary();
		    }
		    else
		    {
			modeYellow = (modeYellow==showTitleEpisode?showEpisode:showTitleEpisode);
			BuildList();
			state = osContinue;
		    }
		    break;
		case kOk:
		    if(HasSubMenu()) 
		    {
			state = cOsdMenu::ProcessKey(Key);
			break;
		    }
		    if (Count())
			return ShowSummary();
		    else
			state = osBack;
		    break;
		default:      
		    break;
	    }
	}
	else
	{
	    if (Key == kOk)
		return Play();
	    else
		state = osContinue;
	}
    }
    if (!HasSubMenu()) 
    {
	if ((HadSubMenu || gl_TimerProgged) && Update())
	{
	    if (gl_TimerProgged) // when using epgsearch's timer edit menu, update is delayed because of SVDRP
	    {
		gl_TimerProgged = 0;
		SetHelpKeys();
	    }
	    Display();
	}
	if (Key != kNone)
	    SetHelpKeys();
	if (gl_InfoConflict)
	{
	    gl_InfoConflict = 0;
	    if (Interface->Confirm(tr("Timer conflict! Show?")))
		return AddSubMenu(new cMenuConflictCheck());
	}      
    }
    return state;
}


