/*
 * quickepgsearch.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */
#include <string>
#include <list>
#include <vdr/plugin.h>
#include <vdr/i18n.h>
#include "services.h"

static const char VERSION[]        = "0.0.1";
static const char DESCRIPTION[]    = "Quick search for broadcasts";
static const char MAINMENUENTRY[]  = "Quick search";
int showInMain=1;


const tI18nPhrase Phrases[] = {
  { "Quick search for broadcasts",
    "Schnelle Suche nach Sendungen",// Deutsch
    "",// TODO 
    "",// TODO
    "",// TODO
    "",// TODO 
    "",// TODO
    "",// TODO 
    "Pikahaku ohjelmaoppaalle",// suomi
    "",// TODO Polski
    "",// TODO Espaol
    "",// TODO  (Greek)
    "",// TODO Svenska
    "",// TODO Romaneste
    "",// TODO Magyar
    "",// TODO Catal
    "",// TODO  (Russian)
    "",// TODO Hrvatski (Croatian)
    "", // Eesti
    "", // Dansk
    "", // Czech
  },
  { "Quick search",
    "Schnellsuche",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "Pikahaku",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "", // Eesti
    "", // Dansk
    "", // Czech
  },
  { "This version of EPGSearch does not support this service!",
    "Diese Version von EPGSearch untersttzt diesen Dienst nicht!",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "EPGSearch-laajennos ei tarjoa vaadittavaa palvelua!",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "", // Eesti
    "", // Dansk
    "", // Czech
  },
  { "EPGSearch does not exist!",
    "EPGSearch nicht vorhanden!",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "EPGSearch-laajennosta ei lydy!",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "", // Eesti
    "", // Dansk
    "", // Czech
  },
  { "Plugin needs vdr-1.3.30 or above!",
    "Plugin bentigt vdr-1.3.30 oder hher!",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "Laajennos tarvitsee toimiakseen vhintn vdr-1.3.30!",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "", // Eesti
    "", // Dansk
    "", // Czech
  },
  { "Show in main menu",
    "Im Hauptmen anzeigen",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "Nyt valinta pvalikossa",
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "",// TODO
    "", // Eesti
    "", // Dansk
    "", // Czech
  },  { NULL }
  };
  
class cMenuQuicksearchSetup : public cMenuSetupPage {
  private:
    virtual void Setup(void)
	{
	    Add(new cMenuEditBoolItem(tr("Show in main menu"), &showInMain, tr("no"), tr("yes")));
	}
  protected:
    virtual void Store(void)
	{
	    SetupStore("QuickEPGSearchInMain", showInMain);
	}
  public:
    cMenuQuicksearchSetup(void)
	{
	    Setup();
	};
  };


class cPluginQuicksearch : public cPlugin {
private:
public:
  cPluginQuicksearch(void);
  virtual ~cPluginQuicksearch();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return tr(DESCRIPTION); }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Stop(void);
  virtual void Housekeeping(void);
  virtual const char *MainMenuEntry(void) { if (showInMain) return tr(MAINMENUENTRY); else return NULL; }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  };

cPluginQuicksearch::cPluginQuicksearch(void)
{
}

cPluginQuicksearch::~cPluginQuicksearch()
{
}

const char *cPluginQuicksearch::CommandLineHelp(void)
{
  return NULL;
}

bool cPluginQuicksearch::ProcessArgs(int argc, char *argv[])
{
  return true;
}

bool cPluginQuicksearch::Initialize(void)
{
  RegisterI18n(Phrases);
  return true;
}

bool cPluginQuicksearch::Start(void)
{
  return true;
}

void cPluginQuicksearch::Stop(void)
{
}

void cPluginQuicksearch::Housekeeping(void)
{
}

cOsdObject *cPluginQuicksearch::MainMenuAction(void)
{
    cPlugin *p = cPluginManager::GetPlugin("epgsearch");
    if (p)
    {
 	Epgsearch_quicksearch_v1_0* serviceData = new Epgsearch_quicksearch_v1_0;
	serviceData->pSearchMenu = NULL;
	
	cOsdMenu* pMenu = NULL;
	if (p->Service("Epgsearch-quicksearch-v1.0", serviceData))
	    pMenu = serviceData->pSearchMenu;
	else
	    Skins.Message(mtError, tr("This version of EPGSearch does not support this service!"));
	
	delete serviceData;
	
	if (pMenu)
	    return pMenu;	
	else
	    return NULL;
    }
    else
    {
	Skins.Message(mtError, tr("EPGSearch does not exist!"));
	return NULL;
    }

  return NULL;
}

bool cPluginQuicksearch::SetupParse(const char *Name, const char *Value)
{
  if (!strcasecmp(Name, "QuickEPGSearchInMain")) showInMain = atoi(Value);
  return true;
}

cMenuSetupPage *cPluginQuicksearch::SetupMenu(void)
{
  return new cMenuQuicksearchSetup;
}

VDRPLUGINCREATOR(cPluginQuicksearch); // Don't touch this!
