#include <string>
#include <list>
#include <vdr/plugin.h>
#include "epgsearchservices.h"
#include "epgsearchext.h"
#include "epgsearchcfg.h"
#include "searchtimer_thread.h"

std::list<std::string> cEpgsearchServiceHandler::SearchTimerList()
{
    std::list<std::string> list;
    cMutexLock SearchExtsLock(&SearchExts);
    for (int i = 0; i < SearchExts.Count(); i++) 
    {
	cSearchExt* search = SearchExts.Get(i);
	if (search)
	    list.push_back(search->ToText());
    }
    return list;    
}

int cEpgsearchServiceHandler::AddSearchTimer(const std::string& settings)
{
    cSearchExt* search = new cSearchExt;
    if (search->Parse(settings.c_str()))
    {
	search->ID = SearchExts.GetNewID();
	LogFile.Log(1,"added search '%s' (%d) via service interface", search->search, search->ID);
	cMutexLock SearchExtsLock(&SearchExts);
	SearchExts.Add(search);
	SearchExts.Save();
	if (search->useAsSearchTimer && !EPGSearchConfig.useSearchTimers) // enable search timer thread if necessary
	    cSearchTimerThread::Init((cPluginEpgsearch*) cPluginManager::GetPlugin("epgsearch"), true);
	return search->ID;
    }
    else
    {
	delete search;
	return -1;
    }
}

bool cEpgsearchServiceHandler::ModSearchTimer(const std::string& settings)
{
    cSearchExt* search = new cSearchExt;
    if (search->Parse(settings.c_str()))
    {
	cSearchExt *searchTemp = SearchExts.GetSearchFromID(search->ID);
	if (searchTemp)
	{
	    searchTemp->Parse(settings.c_str());
	    LogFile.Log(1,"modified search '%s' (%d) via service interface", searchTemp->search, searchTemp->ID);
	    SearchExts.Save();
	    if (searchTemp->useAsSearchTimer && !EPGSearchConfig.useSearchTimers) // enable search timer thread if necessary
		cSearchTimerThread::Init((cPluginEpgsearch*) cPluginManager::GetPlugin("epgsearch"), true);	    
	    return true;
	}
    }
    delete search;
    return false;
}

bool cEpgsearchServiceHandler::DelSearchTimer(int ID)
{
    cSearchExt *search = SearchExts.GetSearchFromID(ID);
    if (search)
    {
	LogFile.Log(1,"search '%s' deleted via service interface", search->search);
	cMutexLock SearchExtsLock(&SearchExts);
	SearchExts.Del(search);
	SearchExts.Save();
	RecsDone.RemoveSearchID(ID);
	return true;
    }
    else
	return false;
}

std::list<std::string> cEpgsearchServiceHandler::QuerySearchTimer(int ID)
{
    std::list<std::string> list;

    cSearchResults* pCompleteSearchResults = NULL;
    cSearchExt* search = SearchExts.GetSearchFromID(ID);
    if (!search) return list;

    pCompleteSearchResults = search->Run();
    if (pCompleteSearchResults)
    {
	// transfer to result list
	pCompleteSearchResults->SortBy(CompareEventTime);
	cSearchResult *result = pCompleteSearchResults->First();
	while (result && result->search) 
	{
	    const cEvent* pEvent = result->event;
	    cTimer* Timer = new cTimer(pEvent);
	    char *cmdbuf = NULL;
		
	    static char bufStart[25];
	    static char bufEnd[25];
	    
	    struct tm tm_r;    time_t eStart = pEvent->StartTime();
	    time_t eStop = pEvent->EndTime();
	    time_t start = eStart - (result->search->MarginStart * 60);
	    time_t stop  = eStop + (result->search->MarginStop * 60);
	    if (result->search->useVPS && pEvent->Vps() && Setup.UseVps)
	    {
		start = pEvent->Vps();
		stop = start + pEvent->Duration();
	    }
	    
	    strftime(bufStart, sizeof(bufStart), "%H%M", localtime_r(&start, &tm_r));
	    strftime(bufEnd, sizeof(bufEnd), "%H%M", localtime_r(&stop, &tm_r));
	    
	    int timerMatch;
	    bool hasTimer = false;
	    if (Timers.GetMatch(pEvent, &timerMatch)) 
		hasTimer = (timerMatch == tmFull);
		
	    if (!result->search->useAsSearchTimer)
		result->needsTimer = false;
		
	    cChannel *channel = Channels.GetByChannelID(pEvent->ChannelID(), true);
	    int timerMode = hasTimer?1:(result->needsTimer?2:0);

	    std::string title = pEvent->Title()?ReplaceAll(pEvent->Title(), "|", "!^pipe!^"):"";
	    title = ReplaceAll(title, ":", "|");
	    std::string shorttext = pEvent->ShortText()?ReplaceAll(pEvent->ShortText(), "|", "!^pipe!^"):"";
	    shorttext = ReplaceAll(shorttext, ":", "|");

	    asprintf(&cmdbuf, "%d:%u:%s:%s:%ld:%ld:%s:%ld:%ld:%s:%d", 
			 result->search->ID,
			 pEvent->EventID(),
			 title.c_str(),
			 shorttext.c_str(),
			 pEvent->StartTime(),
			 pEvent->EndTime(),
			 CHANNELSTRING(channel),
			 timerMode>0?start:-1,
			 timerMode>0?stop:-1,
			 timerMode>0?result->search->BuildFile(pEvent):"",
			 timerMode);				 
		
	    list.push_back(cmdbuf);
	    free(cmdbuf);
	    delete(Timer);
	    result = pCompleteSearchResults->Next(result);
	}
    }
    return list;    
}
